@@ -1,4 +1,5 @@
 #!/usr/bin/env python
+
 import re
 import os
 import sys
@@ -10,17 +11,23 @@ from optparse import OptionParser, OptionGroup
 
 from PIL import Image as PImage
 
+__version__ = '0.2'
 
 TRANSPARENT = (255, 255, 255, 0)
+
 CONFIG_FILENAME = 'sprite.conf'
+ORDERINGS = ['maxside', 'width', 'height', 'area']
+VALID_IMAGE_EXTENSIONS = ['png', 'jpg', 'jpeg', 'gif']
+
 DEFAULT_SETTINGS = {'padding': '0',
-                    'algorithm': 'maxside',
+                    'algorithm': 'square',
+                    'ordering': 'maxside',
                     'namespace': 'sprite',
                     'crop': False,
                     'url': '',
                     'less': False,
                     'optipng': False,
-                    'ignore_filename_paddings': True,
+                    'ignore_filename_paddings': False,
                     'size': True,
                     'png8': False}
 
@@ -40,12 +47,17 @@ class NoSpritesFoldersFoundError(Exception):
     pass
 
 
-class InvalidImageOrderingAlgorithmError(Exception):
+class InvalidImageAlgorithmError(Exception):
     """Raised if the provided algorithm name is invalid."""
     pass
 
 
-class Node(object):
+class InvalidImageOrderingError(Exception):
+    """Raised if the provided ordering is invalid."""
+    pass
+
+
+class SquareAlgorithmNode(object):
 
     def __init__(self, x=0, y=0, width=0, height=0, used=False,
                  down=None, right=None):
@@ -115,10 +127,10 @@ class Node(object):
         self.x = self.y = 0
         self.width += width
         self.down = old_self
-        self.right = Node(x=old_self.width,
-                          y=0,
-                          width=width,
-                          height=self.height)
+        self.right = SquareAlgorithmNode(x=old_self.width,
+                                         y=0,
+                                         width=width,
+                                         height=self.height)
 
         node = self.find(self, width, height)
         if node:
@@ -136,10 +148,10 @@ class Node(object):
         self.x = self.y = 0
         self.height += height
         self.right = old_self
-        self.down = Node(x=0,
-                         y=old_self.height,
-                         width=self.width,
-                         height=height)
+        self.down = SquareAlgorithmNode(x=0,
+                                        y=old_self.height,
+                                        width=self.width,
+                                        height=height)
 
         node = self.find(self, width, height)
         if node:
@@ -154,26 +166,83 @@ class Node(object):
         :param height: New node height.
         """
         node.used = True
-        node.down = Node(x=node.x,
-                               y=node.y + height,
-                               width=node.width,
-                               height=node.height - height)
-        node.right = Node(x=node.x + width,
-                                y=node.y,
-                                width=node.width - width,
-                                height=height)
+        node.down = SquareAlgorithmNode(x=node.x,
+                                        y=node.y + height,
+                                        width=node.width,
+                                        height=node.height - height)
+        node.right = SquareAlgorithmNode(x=node.x + width,
+                                         y=node.y,
+                                         width=node.width - width,
+                                         height=height)
         return node
 
 
-class Image(object):
+class SquareAlgorithm(object):
 
-    ORDERINGS = ['maxside', 'width', 'height', 'area']
+    def process(self, sprite):
+
+        root = SquareAlgorithmNode(width=sprite.images[0].absolute_width,
+                    height=sprite.images[0].absolute_height)
+
+        # Loot all over the images creating a binary tree
+        for image in sprite.images:
+            node = root.find(root, image.absolute_width, image.absolute_height)
+            if node:  # Use this node
+                node = root.split(node, image.absolute_width,
+                                        image.absolute_height)
+            else:  # Grow the canvas
+                node = root.grow(image.absolute_width, image.absolute_height)
+
+            image.x = node.x
+            image.y = node.y
+
+
+class VerticalAlgorithm(object):
+
+    def process(self, sprite):
+        y = 0
+        for image in sprite.images:
+            image.x = 0
+            image.y = y
+            y += image.absolute_height
+
+
+class HorizontalAlgorithm(object):
+
+    def process(self, sprite):
+        x = 0
+        for image in sprite.images:
+            image.y = 0
+            image.x = x
+            x += image.absolute_width
+
+
+class DiagonalAlgorithm(object):
+
+    def process(self, sprite):
+        x = y = 0
+        for image in sprite.images:
+            image.x = x
+            image.y = y
+            x += image.absolute_width
+            y += image.absolute_height
+
+
+ALGORITHMS = {'square': SquareAlgorithm,
+              'vertical': VerticalAlgorithm,
+              'horizontal': HorizontalAlgorithm,
+              'diagonal': DiagonalAlgorithm}
+
+
+class Image(object):
 
     def __init__(self, name, sprite):
         """Image constructor
 
         :param name: Image name.
         :param sprite: :class:`~Sprite` instance for this image."""
+        self.x = None
+        self.y = None
         self.name = name
         self.sprite = sprite
         self.filename, self.format = name.rsplit('.', 1)
@@ -196,10 +265,8 @@ class Image(object):
             self._crop_image()
 
         self.width, self.height = self.image.size
-
-        self.width += self.padding[1] + self.padding[3]
-        self.height += self.padding[0] + self.padding[2]
-        self.node = None
+        self.absolute_width = self.width + self.padding[1] + self.padding[3]
+        self.absolute_height = self.height + self.padding[0] + self.padding[2]
 
     def _crop_image(self):
         """Crop the image searching for the smallest possible bounding box
@@ -235,7 +302,9 @@ class Image(object):
         if type(padding) == str:
             padding = padding.replace('px', '').split()
 
-        if len(padding) == 3:
+        if len(padding) == 4:
+            padding = padding
+        elif len(padding) == 3:
             padding = padding + [padding[1]]
         elif len(padding) == 2:
             padding = padding * 2
@@ -268,6 +337,7 @@ class Image(object):
                 padding_info_name = '_%s' % padding_info_name
             name = name[:len(padding_info_name) * -1 or None]
         name = re.sub(r'[^\w\-_]', '', name)
+
         return '%s-%s' % (self.sprite.namespace, name)
 
     @property
@@ -295,80 +365,65 @@ class Image(object):
             padding = self.sprite.config.padding
         return self._generate_padding(padding)
 
-    @property
-    def x(self):
-        """Y coordinate for this image."""
-        return self.node.x + self.padding[3]
-
-    @property
-    def y(self):
-        """X coordinate for this image."""
-        return self.node.y + self.padding[0]
-
     def __lt__(self, img):
         """Use maxside, width, height or area as ordering algorithm.
 
         :param img: Another :class:`~Image`."""
-        algorithm = self.sprite.config.algorithm
-        if algorithm == 'width':
-            return self.width <= img.width
-        elif algorithm == 'height':
-            return self.height <= img.height
-        elif algorithm == 'area':
-            return self.width * self.height <= img.width * img.height
+        ordering = self.sprite.config.ordering
+        ordering = ordering[1:] if ordering.startswith('-') else ordering
+
+        if ordering not in ORDERINGS:
+            raise InvalidImageOrderingError(ordering)
+
+        if ordering == 'width':
+            return self.absolute_width <= img.absolute_width
+        elif ordering == 'height':
+            return self.absolute_height <= img.absolute_height
+        elif ordering == 'area':
+            return self.absolute_width * self.absolute_height <= \
+                   img.absolute_width * img.absolute_height
         else:
-            return max(self.width, self.height) <= max(img.width, img.height)
+            return max(self.absolute_width, self.absolute_height) <= \
+                   max(img.absolute_width, img.absolute_height)
 
 
 class Sprite(object):
 
-    DEFAULT_SETTINGS = {'padding': '0',
-                        'algorithm': 'maxside',
-                        'namespace': 'sprite',
-                        'crop': False,
-                        'url': ''}
-
     def __init__(self, name, path, manager):
         """Sprite constructor.
 
         :param name: Sprite name.
         :param path: Sprite path
-        :param manager: Sprite manager. :class:`~MultipleSpriteManager` or
+        :param manager: Sprite manager. :class:`~ProjectSpriteManager` or
                         :class:`SimpleSpriteManager`"""
         self.name = name
         self.manager = manager
         self.images = []
         self.path = path
+        self.cachebuster_hash = ''
 
         self.config = manager.config.extend(get_file_config(self.path))
-
-        if self.config.algorithm not in Image.ORDERINGS:
-            raise InvalidImageOrderingAlgorithmError(self.config.algorithm)
-
         self.process()
 
     def process(self):
         """Process a sprite path searching for all the images and then
         allocate all of them in the most appropriate position.
         """
-        self.images = self._locate_images()
-        width = self.images[0].width
-        height = self.images[0].height
-        root = Node(width=width, height=height)
 
-        # Loot all over the images creating a binary tree
-        for image in self.images:
-            self.manager.log("\t %s => .%s" % (image.name, image.class_name))
-            node = root.find(root, image.width, image.height)
-            if node:  # Use this node
-                image.node = root.split(node, image.width, image.height)
-            else:  # Grow the canvas
-                image.node = root.grow(image.width, image.height)
+        algorithm = ALGORITHMS.get(self.config.algorithm)
+
+        if not algorithm:
+            raise InvalidImageAlgorithmError(self.config.algorithm)
+
+        self.algorithm = algorithm()
+        self.images = self._locate_images()
+
+        self.algorithm.process(self)
 
     def _locate_images(self):
         """Return all valid images within a folder.
 
-        All files with a extension not included in VALID_IMAGE_EXTENSIONS
+        All files with a extension not included i
         (png, jpg, jpeg and gif) or beginning with '.' will be ignored.
 
         If the folder doesn't contain any valid image it will raise
@@ -377,7 +432,7 @@ class Sprite(object):
         The list of images will be ordered using the desired ordering
         algorithm. The default is 'maxside'.
         """
-        extensions = '|'.join(self.manager.VALID_IMAGE_EXTENSIONS)
+        extensions = '|'.join(VALID_IMAGE_EXTENSIONS)
         extension_re = re.compile('.+\.(%s)$' % extensions, re.IGNORECASE)
 
         images = [Image(n, sprite=self) for n in os.listdir(self.path) if \
@@ -393,7 +448,10 @@ class Sprite(object):
             dup = [i for i in images if class_names.count(i.class_name) > 1]
             raise MultipleImagesWithSameNameError(dup)
 
-        return sorted(images, reverse=True)
+        for image in images:
+            self.manager.log("\t %s => .%s" % (image.name, image.class_name))
+
+        return sorted(images, reverse=self.config.ordering[0] != '-')
 
     def save_image(self):
         """Create the image file for this sprite."""
@@ -405,8 +463,8 @@ class Sprite(object):
         width = height = 0
 
         for image in self.images:
-            x = image.node.x + image.width
-            y = image.node.y + image.height
+            x = image.x + image.absolute_width
+            y = image.y + image.absolute_height
             if width < x:
                 width = x
             if height < y:
@@ -417,7 +475,12 @@ class Sprite(object):
 
         # Paste the images inside the canvas
         for image in self.images:
-            canvas.paste(image.image, (image.x, image.y))
+            canvas.paste(image.image, (image.x + image.padding[3],
+                                       image.y + image.padding[0]))
+
+        if self.config.cachebuster or self.config.cachebuster_filename:
+            self.cachebuster_hash = hashlib.sha1(canvas.tostring()
+                                                ).hexdigest()[:6]
 
         # Save png
         sprite_filename = '%s.png' % self.filename
@@ -471,22 +534,22 @@ class Sprite(object):
         class_names = ',\n'.join(class_names)
 
         # create an unique style for all the sprites for less bloat
-        style = "%s{background-image:url('%s');background-repeat:no-repeat;}\n"
+        style = "%s{background-image:url(%s);background-repeat:no-repeat;}\n"
         css_file.write(style % (class_names, self.image_url))
 
         for image in self.images:
             data = {'image_class_name': image.class_name,
-                    'top': image.node.y * -1 if image.node.y else 0,
-                    'left': image.node.x * -1 if image.node.x else 0,
-                    'width': image.width,
-                    'height': image.height}
+                    'top': image.y * -1 if image.y else 0,
+                    'left': image.x * -1 if image.x else 0,
+                    'width': image.absolute_width,
+                    'height': image.absolute_height}
 
             style = (".%(image_class_name)s{"
                      "background-position:%(left)ipx %(top)ipx;")
 
             if self.config.size:
                 # if it's required add the image size to the sprite
-                style += "width:%(width)spx; height:%(height)spx;"
+                style += "width:%(width)spx;height:%(height)spx;"
 
             style += "}\n"
 
@@ -497,11 +560,15 @@ class Sprite(object):
     @property
     def namespace(self):
         """Return the namespace for this sprite."""
+        if not self.config.namespace:
+            return self.name
         return '%s-%s' % (self.config.namespace, self.name)
 
     @property
     def filename(self):
         """Return the desired filename for files generated by this sprite."""
+        if self.config.cachebuster_filename:
+            return '%s_%s' % (self.name, self.cachebuster_hash)
         return self.name
 
     @property
@@ -514,14 +581,13 @@ class Sprite(object):
     def image_url(self):
         """Return the sprite image url."""
         url = os.path.relpath(self.image_path, self.manager.output_path('css'))
+
         if self.config.url:
             url = os.path.join(self.config.url, '%s.png' % self.filename)
 
         if self.config.cachebuster:
-            sprite_file = open(self.image_path, 'rb')
-            sprite_hash = hashlib.sha1(sprite_file.read()).hexdigest()
-            sprite_file.close()
-            url = "%s?%s" % (url, sprite_hash[:6])
+            url = "%s?%s" % (url, self.cachebuster_hash)
+
         return url
 
 
@@ -571,8 +637,6 @@ class ConfigManager(object):
 
 class BaseManager(object):
 
-    VALID_IMAGE_EXTENSIONS = ['png', 'jpg', 'jpeg', 'gif']
-
     def __init__(self, path, config, output=None):
         """BaseManager constructor.
 
@@ -627,7 +691,7 @@ class BaseManager(object):
         raise NotImplementedError()
 
 
-class MultipleSpriteManager(BaseManager):
+class ProjectSpriteManager(BaseManager):
 
     def process(self):
         """Process a path searching for folders that contain images.
@@ -657,11 +721,14 @@ class MultipleSpriteManager(BaseManager):
 
         If two images have the same name,
         :class:`~MultipleImagesWithSameNameError` will be raised.
+
+        This is not the default manager. It is only used if you use
+        the ``--project`` argument.
         """
         for sprite_name in os.listdir(self.path):
             # Only process folders
             path = os.path.join(self.path, sprite_name)
-            if os.path.isdir(path):
+            if os.path.isdir(path) and not sprite_name.startswith('.'):
                 self.process_sprite(path=path, name=sprite_name)
 
         if not len(self.sprites):
@@ -674,10 +741,9 @@ class SimpleSpriteManager(BaseManager):
 
     def process(self):
         """Process a single folder and create one sprite. It works the
-        same way as :class:`~MultipleSpriteManager`, but only for one folder.
+        same way as :class:`~ProjectSpriteManager`, but only for one folder.
 
-        This is not the default manager. It is only used if you use
-        the ``--simple`` default argument.
+        This is the default manager.
         """
         self.process_sprite(path=self.path, name=os.path.basename(self.path))
         self.save()
@@ -761,8 +827,8 @@ PImage.Image.load = patched_load
 def main():
     parser = OptionParser(usage=("usage: %prog [options] source_dir [<output> "
                                  "| --css=<dir> --img=<dir>]"))
-    parser.add_option("-s", "--simple", action="store_true", dest="simple",
-                      help="generate sprites for a single folder")
+    parser.add_option("--project", action="store_true",
+                  dest="project", help="generate sprites for multiple folders")
     parser.add_option("-c", "--crop", dest="crop", action='store_true',
                 help="crop images removing unnecessary transparent margins")
     parser.add_option("-l", "--less", dest="less", action='store_true',
@@ -775,6 +841,8 @@ def main():
                       help="force this padding in all images")
     parser.add_option("-z", "--no-size", action="store_false", dest="size",
                       help="don't add the image size to the sprite")
+    parser.add_option("-v", "--version", action="store_true", dest="version",
+                      help="show program's version number and exit")
 
     group = OptionGroup(parser, "Output Options")
     group.add_option("--css", dest="css_dir", default='',
@@ -785,7 +853,10 @@ def main():
 
     group = OptionGroup(parser, "Advanced Options")
     group.add_option("-a", "--algorithm", dest="algorithm", default=None,
-                    help=("ordering algorithm: maxside, width, height or "
+                    help=("allocation algorithm: square, vertical, horizontal "
+                          "(default: square)"), metavar='NAME')
+    group.add_option("--ordering", dest="ordering", default=None,
+                    help=("ordering criteria: maxside, width, height or "
                           "area (default: maxside)"), metavar='NAME')
     group.add_option("--namespace", dest="namespace",  default=None,
                       help="namespace for all css classes (default: sprite)")
@@ -808,13 +879,25 @@ def main():
     group = OptionGroup(parser, "Browser Cache Invalidation Options")
     group.add_option("--cachebuster", dest="cachebuster",
                     action='store_true',
-                    help=("use the sprite's first 6 sha1 characters as a "
+                    help=("use the sprite's sha1 first 6 characters as a "
                           "queryarg everytime that file is referred from "
                           "the css"))
+    group.add_option("--cachebuster-filename", dest="cachebuster_filename",
+                    action='store_true',
+                    help=("append the sprite's sha first 6 characters "
+                          "to the otput filename"))
     parser.add_option_group(group)
 
     (options, args) = parser.parse_args()
 
+    if options.version:
+        sys.stdout.write("%s\n" % __version__)
+        sys.exit(0)
+
+    if options.cachebuster and options.cachebuster_filename:
+        parser.error("You can't use --cachebuster and "
+                     "--cachebuster-filename at the same time.")
+
     if not len(args):
         parser.error("You must provide the folder containing the sprites.")
 
@@ -837,10 +920,10 @@ def main():
     if not os.path.isdir(source):
         parser.error("Directory not found: '%s'" % source)
 
-    if options.simple:
-        manager_cls = SimpleSpriteManager
+    if options.project:
+        manager_cls = ProjectSpriteManager
     else:
-        manager_cls = MultipleSpriteManager
+        manager_cls = SimpleSpriteManager
 
     # Get configuration from file
     config = get_file_config(source)
@@ -861,6 +944,10 @@ def main():
         sys.stderr.write("Error: No images found.\n")
     except NoSpritesFoldersFoundError:
         sys.stderr.write("Error: No sprites folders found.\n")
+    except InvalidImageOrderingError, e:
+        sys.stderr.write("Error: Invalid image ordering %s.\n" % e.args[0])
+    except InvalidImageAlgorithmError, e:
+        sys.stderr.write("Error: Invalid image algorithm %s.\n" % e.args[0])
 
 
 if __name__ == "__main__":
